#define SERIAL_PRINTF_MAX_BUFF  (256)

#include <stdarg.h>                                                     //Only required if serialPrintln is used.
#include "StepMotorDriverA4988.h"




void   serialPrintln (const char* fmt, ...);
int8_t timerUpdate (uint32_t timTop32);




void
setup ()
{
  Serial.begin (115200);                                                //Only required if serialPrintln is used.
  serialPrintln ("Starting %s\r\n", __FILE__);
  serialPrintln ("Compile Date %s\r\n", __DATE__);
  #ifdef MOTOR_TIMER_5
    stepMotor5Reset ();                                                 //Send a 1ms reset pulse to the controller.
  #endif
  stepMotorInit ();
  #ifdef MOTOR_TIMER_5
    stepMotor5WakeUp ();
  #endif
}





void
serialPrintln (const char* fmt,                                         //https://medium.com/@kslooi/print-formatted-data-in-arduino-serial-aaea9ca840e3
               ...)
{
  char buff[SERIAL_PRINTF_MAX_BUFF];                                    //pointer to the variable arguments list.
  memset (buff, 0, SERIAL_PRINTF_MAX_BUFF);
  va_list pargs;                                                        //Initialise pargs to point to the first optional argument
  va_start (pargs, fmt);                                                //create the formatted data and store in buff
  vsnprintf (buff, SERIAL_PRINTF_MAX_BUFF, fmt, pargs);
  va_end (pargs);
  Serial.println (buff);
}






void
loop ()
{
  bool once = true;
  unsigned int steps_rotation = 200;
  static int rpm = 0;
  if (once != false)
  {
    #ifdef MOTOR_TIMER_1
      serialPrintln ("TimerA set returns %i", stepMotor1TimerUpdate (10000UL));//1.255ms.
    #endif
    #ifdef MOTOR_TIMER_3
      serialPrintln ("TimerB set returns %i", stepMotor3TimerUpdate (20000UL));//2.5ms.
    #endif
    #ifdef MOTOR_TIMER_4
      serialPrintln ("TimerC set returns %i", stepMotor4TimerUpdate (40000UL));//5ms.
    #endif
    #ifdef MOTOR_TIMER_5
      serialPrintln ("TimerD set returns %i", stepMotor5TimerUpdate (80000UL));//10ms.
    #endif
    once = false;
    serialPrintln ("RPM:10, Step/Rot:%i, %li", steps_rotation, motorTimerRPMtoTimerTop32 (10., steps_rotation));
    serialPrintln ("RPM:321, Step/Rot:%i, %li", steps_rotation, motorTimerRPMtoTimerTop32 (321., steps_rotation));
    serialPrintln ("RPM:78, Step/Rot:%i, %li", steps_rotation, motorTimerRPMtoTimerTop32 (78., steps_rotation));
    once = true;
  }
  for (;;)
  {
    float rpmReal = (float) (rpm);
    unsigned long steps_s = motorTimerRPMtoTimerTop32 (rpmReal, steps_rotation);
    serialPrintln ("RPM: %i, Steps/s: %li", rpm, steps_s);
    #ifdef MOTOR_TIMER_5
      stepMotor5TimerUpdate (steps_s);
    #endif
    ++rpm;
    if (rpm > 250) rpm = 0;
    delay (1000);
  }
}


